<?php
namespace Geral\Logic;

use \Geral\Entities\Filial;
use Geral\Repositories\Perfacessparc;

/**
 *
 */
class Login
{

    private $conn;

    private $email;
    private $senha;
    private $cdfilial;
    private $nrorganizacao;
    private $nrestrutura;

    /** @var $organizacao \Geral\Entities\Organizacao */
    private $organizacao;
    /** @var $filial \Geral\Entities\Filial*/
    private $filial;
    /** @var $operador \Geral\Entities\Operador */
    private $operador;
    /** @var $estrutura \Geral\Entities\Estruturam */
    private $estrutura;
    /** @var  $perfilacesso \Geral\Entities\Perfacessparc */
    private $perfilacesso;

    /**
     * Construtor.
     *
     * @param $email
     * @param $senha
     * @param $cdfilial
     * @param $nmorganizacao
     */
    public function __construct($email, $senha, $cdfilial = null, $nmorganizacao = null)
    {
        $this->email         = $email;
        $this->senha         = $senha;
        $this->cdfilial      = $cdfilial;
        $this->nrorganizacao = $nmorganizacao;
    }

    /**
     * Inicializa os objetos de orgnaizacao, filial e operador, alem de validar os dados do usuario.
     *
     * @return bool
     * @throws \Exception
     */
    public function valida()
    {
        $this->loginVirtualPrivateDatabase();
        
        $this->obterOperador();
        $this->validaOperadorESenha();
        $this->obterParametrosDeTrabalho();
        $this->validaParametrosDeTrabalho();
        $this->obterEstrutura();
        $this->obterOrganizacao();

        if ($this->usaControleDeAcesso()) {
            $this->obterPerfilAcesso();
            $this->validaPerfilAcesso();
        };
        if (\Setforms2\Shared\Config::getInstance()->getProjectUsafilial() == 'true') {
            $this->obterFilial();
            $this->validaFilial();
        }
        return true;
    }

    /**
     * Retorna o objeto de operador.
     *
     * @return \Geral\Entities\Operador
     */
    public function getOperador()
    {
        return $this->operador;
    }

    /**
     * Retorna o objeto de filial.
     *
     * @return \Geral\Entities\Filial
     */
    public function getFilial()
    {
        return $this->filial;
    }

    /**
     * Retorna o objeto de organizacao.
     *
     * @return \Geral\Entities\Organizacao
     */
    public function getOrganizacao()
    {
        return $this->organizacao;
    }

    /**
     * @throws \Geral\Exceptions\OrganizacaoNaoEncontrada
     * @return \Geral\Entities\Organizacao
     */
    private function obterOrganizacao()
    {
        $this->organizacao = $this->operador->getOrgtrab();
    }

    /**
     * Retorna o objeto de filial a partir do codigo da filial. Caso nao encontre uma filial, retorna uma mensagem de
     * erro.
     *
     * @throws \Geral\Exceptions\FilialNaoEncontrada
     * @return Filial
     */
    private function obterFilial()
    {
        if(is_object($this->estrutura)){
            $this->filial = Filial::findOneByCdfilialAndNrorg(
                $this->estrutura->getCdintestrutura(), $this->organizacao->getNrorg()
            );
        }
    }

    private function validaFilial() {
        if (!$this->filial) {
            throw new \Geral\Exceptions\FilialNaoEncontrada();
        }
    }

    /**
     * Retorna o objeto de operador a partir do email
     *
     * @throws Login\Exceptions\OperadorNaoEncontrado
     */
    private function obterOperador()
    {
        /** @var $operador \Geral\Entities\Operador */
        $this->operador = \Geral\Entities\Operador::findOneByDsemailoper($this->email);
    }

    /**
     * Valida o operador, atualiza a senha se for primeiro login, verifica os parametros do operador.
     *
     * @throws \LoginException
     * @throws \Geral\Exceptions\OperadorNaoEncontrado
     */
    private function validaOperadorESenha()
    {
        if (!$this->operador) {
            throw new \Geral\Logic\Login\Exceptions\OperadorNaoEncontrado();
        }

        
        if($this->operador->getIdoperativo() != 'A') {
            throw new \Geral\Logic\Login\Exceptions\OperadorInativo();
        }
        // Se operador tem senha para o sistema Delphi e nao tem senha WEB, sera esta a senha considerada.
        if ($this->operador->getCdsenhoper() && !$this->operador->getCdsenhaoperweb()) {
            if ($msg = $this->_validaOperadorWebService()) {
                
                throw new \Geral\Logic\Login\Exception($msg);
            }
        } else if ($this->_validaAlteraSenha() === false) {
            throw new \Geral\Logic\Login\Exceptions\OperadorSenhaInvalida();
        }
    }

    public function getUserLoginConnection() {
        $host = \Setforms2\Shared\Config::getInstance()->getConnectionHost();
		$serviceName = \Setforms2\Shared\Config::getInstance()->getConnectionServicename();
		$loginUser = \Setforms2\Shared\Config::getInstance()->getConnectionLoginuser();
		$loginPassword = \Setforms2\Shared\Config::getInstance()->getConnectionLoginpassword();
		$loginPassword = \Setforms2\Shared\Util\Util::decrypt($loginPassword);
        if (!$this->conn) {
		   $conn = new \Doctrine2\DBAL\Driver\OCI8\OCI8Connection($loginUser, $loginPassword, "//$host/$serviceName");
		   $this->conn = $conn;
        }
        return $this->conn;
    }

    /**
     * Valida operador usando web service para execucao de metodo Delphi.
     *
     * @throws \Exception
     * @return null|string
     */
    private function _validaOperadorWebService() {
        $url = 'https://www.security.teknisa.com/comwebservice/service.asmx?wsdl';

        $opcoes = array(
            'encoding'    => 'ISO-8859-1',
            'trace'       => true,
            'exceptions'  => true,
            'compression' => SOAP_COMPRESSION_ACCEPT | SOAP_COMPRESSION_GZIP,
        );

        $argumentos = array(
            new \SoapVar($this->operador->getCdoperador(), XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
            new \SoapVar($this->senha, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
        );

        $parametros = array(
            'rdm'    => 'Bib_Html.rdmDBGeralHtm',
            'method' => 'ValidaConexaoHTM',
            'args'   => $argumentos,
        );
        try {
            $client = new \SoapClient($url, $opcoes);
            $retorno = $client->executeCOMMethod($parametros);
            $resultado = $retorno->executeCOMMethodResult;
        } catch (\SoapFault $e) {
            throw new \Exception($e->getMessage());
        }
        if ($resultado === 'SenhaInv') {
            return 'msg_valida_senha_operador'; // Senha Invalida.
        }

        if ($resultado === 'OperNCad') {
            return 'msg_operador_n_cadastrado'; // Operador nao cadastrado.
        }
        return null;
    }

    /**
     *  Valida ou altera senha.
     *
     * @return void|boolean
     */
    private function _validaAlteraSenha(){
        $criptosenha    = md5($this->senha);
        $cdsenhaoperweb = ($this->senha !== '')?$this->operador->getCdsenhaoperweb() : null;

        if (($cdsenhaoperweb === null || $cdsenhaoperweb === 'd41d8cd98f00b204e9800998ecf8427e') && $this->senha !== '') {
            \Operador::alteraSenhaOperador($this->operador->getCdoperador(), $criptosenha);
            return null;
        } else if ($cdsenhaoperweb !== $criptosenha) {
            return false;
        }
        return true;
    }

    /**
     * @throws \Exception
     */
    private function validaParametrosDeTrabalho()
    {
        if (!is_numeric($this->nrorganizacao)) {
            throw new \Exception('Operador sem organização de trabalho parametrizada!');
        }
        if ((\Setforms2\Shared\Config::getInstance()->getProjectParamestrutura() == 'true') && !$this->nrestrutura) {
            throw new \Exception('Operador sem parametro de estrutura!');
        }
    }

    /**
     * @throws \Exception
     */
    private function obterParametrosDeTrabalho()
    {
        $this->nrorganizacao = $this->operador->getNrorgtrab();
        $this->nrestrutura = $this->operador->getNrestruttrab();
    }

    private function obterEstrutura()
    {
        $this->estrutura = $this->operador->getEstruttrab();
    }

    /**
     * @throws \Exception
     * @return mixed
     */
    private function loginVirtualPrivateDatabase()
    {
        $host = \Setforms2\Shared\Config::getInstance()->getConnectionHost();
        if (\Setforms2\Shared\Config::getInstance()->getProjectUsavpd() != 'true') {
            return;
        }
        $operador = $this->getVPDOperadorOrganizacao();
        $this->validaVPDOperador($operador); 

        tk_set_env_var('USUARIO_VPD', 'USR_ORG_' . $this->nrorganizacao);
    }

    /**
     * Verificacao caso nao exista perfilacessoSelecionado.
     */
    private function obterPerfilAcesso()
    {//Alteração feita para que o sistema valide se o operador tem controle de acesso para o projeto.
        //$this->perfilacesso= \Geral\Entities\Perfacessparc::findOneByNrparcnegocioAndNrorg(
            //$this->operador->getNrparcnegocio(), $this->organizacao->getNrorg()
        //);
		
        $config = \Setforms2\Shared\Config::getInstance();
        $sql = 'SELECT NRPERFILACESSO 
                  FROM PERFACESSPARC 
                 WHERE NRPARCNEGOCIO = '.$this->operador->getNrparcnegocio().'
                   AND NRORG = '.$this->organizacao->getNrorg().'
                   AND EXISTS (SELECT * 
                                 FROM TKU_ACCESSCONTROL_RULE
                                WHERE GROUP_ID = NRPERFILACESSO
                                  AND PROJECT_NAME LIKE \''.$config->getProjectName().'\')';

        $perfil = \Setforms2\Shared\Registry::getEntityManager()->getConnection()->fetchAll($sql);
        if (count($perfil)>0) {
        //Alteração feita para que apos o sistema validar se o operador tem controle de acesso para o projeto, o this->perfilacesso volta a ter o valor obitido no findOneBy.
        $this->perfilacesso= \Geral\Entities\Perfacessparc::findOneByNrparcnegocioAndNrorg(
            $this->operador->getNrparcnegocio(), $this->organizacao->getNrorg()
        );
        }
        
    }

    private function validaPerfilAcesso() {
        if (!$this->perfilacesso) {
            throw new \Exception('Operador sem perfil de acesso!');
        }
        
        $this->verificaValidadeAcesso();
    }

    private function usaControleDeAcesso() {
        $paramgeral = \Geral\Entities\Paramgeral::findOneByNrorg($this->organizacao->getNrorg());
        if (!$paramgeral || $paramgeral->getIdutctrlaces() == 'N') {
            return false;
        }
        return true;
    }

    /**
     * @return mixed
     */
    public function getPerfilacesso()
    {
        return $this->perfilacesso;
    }


    /**
     * Verifica validade de acesso e inativa perfil com data de validade expirada.
     *
     * @throws Login\Exceptions\PerfilAcessoinativo
     * @throws Login\Exceptions\DataValidadeAcesso
     */
    private function verificaValidadeAcesso()
    {
        $this->verificaPerfilAtivo();
        if (!$this->verificaDataValidadePerfil()){
            $this->inativaPerfilAcesso();
            throw new \Geral\Logic\Login\Exceptions\DataValidadeAcesso();
        }
    }

    /**
     * Inativa perfil de acesso.
     */
    private function inativaPerfilAcesso() {
        $this->perfilacesso->setIdativo('N');
        $this->perfilacesso->persist();
        \Setforms2\Shared\Registry::getEntityManager()->flush();
    }

    /**
     * @throws Login\Exceptions\PerfilAcessoinativo
     */
    private function verificaPerfilAtivo() {
        if ($this->perfilacesso->getIdativo() === 'N') {
            throw new \Geral\Logic\Login\Exceptions\PerfilAcessoinativo();
        }
    }

    /**
     * Retorna verdadeiro perfil estiver valido.
     *
     * @throws Login\Exceptions\DataValidadeAcesso
     */
    private function verificaDataValidadePerfil() {
        $hoje = new \TkDateTime(date("d/m/Y"));
        $validade = new \TkDateTime($this->perfilacesso->getDtvalperfilparc());
        if (!$validade || $hoje->daysAfter($validade) > 0) {
            return true;
        }
        return false;
    }

    /**
     * @return mixed
     */
    private function getVPDOperadorOrganizacao()
    {
        $conn = $this->getUserLoginConnection();
        $sql = "SELECT CDOPERADOR, NRORGTRAB FROM OPERADOR WHERE DSEMAILOPER = '{$this->email}'";
        $operador = $conn->query($sql)->fetch(\Doctrine2\DBAL\Driver\PDOConnection::FETCH_ASSOC);
        $this->nrorganizacao = $operador['NRORGTRAB'];
        return $operador;
    }

    /**
     * @param $operador
     * @throws \Exception
     */
    private function validaVPDOperador($operador)
    {
        if (!$operador || !$operador['CDOPERADOR']) {
            throw new \Exception('Operador não encontrado.');
        }
        if (!is_numeric($operador['NRORGTRAB'])) {
            throw new \Exception('Organização de trabalho não parametrizada, contacte o administrador.');
        }
    }

    public static function getUsuarioVPD() {
     
        return tk_get_env_var('USUARIO_VPD');
    }

    /**
     * @return mixed
     */
    public function getSenha()
    {
        return $this->senha;
    }

}